#ifndef _EQUIPDES_H_
#define _EQUIPDES_H_ 1

/*	EquipDes.h - last update : 29 avril 1999
	Copyright Acquisition group, GANIL Caen, France

 	contents : structure de description des equipements du systeme VME
		d'acquisition.
		   La table des equipements est compose d'un entete EQUIPHD 
		   suivi d'une table d'equipements dont le nombre d'entree 
		   depend de la taille memoire affectee a la table des 
		   equipements. Chaque entre de la table correspond a un 
		   descripteur EQUIPDES de taille fixe, chaque type 
		   d'equipement etant decrit dans un membre de l'union Desc.

        history : 
	   14 nov  1997 - Frederic Saillant - modifs pour zones memoire
	   22 juin 1998 - Bruno Raine       - modif structure CAMDES
	   23 nov 1998  - Bruno Raine       - modif struct EQUIPDES
                                              (ajout du champs GlobalName)
	   12 jan 1999  - Bruno Raine       - add CSR in CPUDES
	   19 jan 1999  - Bruno raine       - modif CAMDES, VXIDES, VICDES
	   30 mar 1999  - Bruno Raine       - Process types definition 
           29 avr 1999  - Bruno Raine       - D2VBDES definition
*/
#include <semaphore.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Version de table des equipements */
#define EQUIP_VERSION 4


#define EQUIP_NAME_LEN 32    /* Taille des noms d'equipements */
#define EXE_NAME_LEN   80    /* Size of excutables names */

/* Definition des valeurs d'init de l'entete d'une entree EQUIDES */
#define EQ_NOINITFAMI -1     /* famille inconnue ou non initialisee */
#define EQ_NOINITNUM  -1     /* Numero d'equip ds la famille non initialisee */
#define EQ_NOINITID   -1     /* Ident equipement non initialise */
#define EQ_GLOBNAME   -1     /* Nom d'equipment global */
#define EQ_NOSHM      0xffffffff /* Pas de memoire partagee sur un CPU */

/* definition des familles d'equipements */
#define BADFAMI -1
#define CPUFAMI  1		/* famille CPU */
#define ASCUFAMI 2		/* famille ASCU */
#define IFVFAMI  3		/* famille IFV */
#define MEMFAMI  4		/* famille zone memoire */

#define CAMFAMI  6		/* famille CAMAC (MODIFIE LE 10 DEC 90) */
#define VXIFAMI  7		/* famille VXI */
#define VICFAMI  8              /* famille VIC */

#define ECHFAMI  10		/* famille module echelle */
#define	HTFAMI	 11		/* famille des hautes tensions */
#define	CNETFAMI 12		/* famille contoleurs CAENET */
#define MEMEXTFAMI 13		/* coupleur externe d'acq de type memoire */
#define MEXTFAMI MEMEXTFAMI
#define AMBXFAMI 14		/* mailbox acquisition */
#define PROCFAMI 15             /* Famille process */
#define SBUFFAMI PROCFAMI       /* famille serveur de buffers */
#define VMEMEMFAMI  16		/* famille carte memoire VME */
#define D2VBFAMI 17             /* famille interface D2VB */
#define UTMFAMI  80

/* definition des types d'applications ==> A revoir !*/
#define NO_APPL -1             /* Pas d'application particuliere pour ce cpu */
#define DEV   0		       /* developpement */
#define PF    1		       /* prefiltrage */
#define TRA   2 	       /* traitement et control */
#define CAM   3		       /* camac */
#define TRANS 4		       /* transfert vers hote */
#define VXI   5		       /* VXI */
#define CC    6		       /* Commandes et Control d'equipement */


/* definition des types de CPU's */

/* definition des Cpu's FORCE, en principe non pris en compte sous Lynx */
#define F20 0X20			/* FORCE CPU 20 */
#define F21 0X21			/* FORCE CPU 21 */
#define F5  0X05			/* FORCE CPU 5  */
#define F22 0X22			/* FORCE CPU 22 */
#define F23 0X23			/* FORCE CPU 23 */
#define F29 0X29			/* FORCE CPU 29 */
#define F30 0X30			/* FORCE CPU 30 */
#define F31 0X31			/* FORCE CPU 31 */
#define F33 0X33			/* FORCE CPU 33 */
#define F37 0X37			/* FORCE CPU 37 */
#define F40 0X40			/* FORCE CPU 40 */
#define IBC 0x2C                        /* FORCE IBC 20 */

/* Definition des Cpu's PowerPC */
#define R8062 0x8062                    /* CES CPU RIO2 8062 */

/* Definition des types de memoire partagee et valeurs par defaut */
#define MemDefAmCode 0x09  /* AM code par defaut */

#define ShmMem       1 /* Memoire partagee posix locale sur un CPU */
#define ShMemCpu     2 /* Memoire partagee d'un CPU sur le VME*/
#define MemVme       3 /* Memoire accessible par VME a partir du cpu courant */

#define EQMAP_READ   1 /* Option de mapping en read only */
#define EQMAP_RW     2 /* Option de mapping en read/write */

/* Definitions des type de cartes VIC */
#define VIC8250      0x8250
#define VIC8251      0x8251

/* Definition des borne autorisees pour les numeros de chassis VIC */
#define VIC_CRATE_MIN 1
#define VIC_CRATE_MAX 14


/* definition des types de coupleur de branche CAMAC */
#define CBD_8210 0x8210		 /* Coupleur VME */
#define CBD_8110 0x8110		 /* Coupleur VSB */
#define VCC_2117 0x2117          /* Controleur de chassis CAMAC sur bus VIC */

/* definition des bornes autorisees pour les numeros de branche CAMAC */
#define CAM_BRANCH_MIN 0                /* Numero de branche min */
#define CAM_BRANCH_MAX 6                /* Numero de branche max */

/* definition des types de VXI */
#define VXI_Local 0        /* Acces au VXI par le CPU local */
#define VXI_VIC   VIC      /* Acces au VXI via VIC bus */

/* Definition des drivers de lecture des coupleurs externe de type memoire */
#define MextNoFifo  0	/* pas de FIFO, evt GANIL */
#define MextGanFifo 1	/* Fifo de type GANIL, evt GANIL */
#define MextChimera 2	/* pas de FIFO, evt CHIMERA */

/* Definition des options de synchronisation par mailbox pour coupleur
   externe de type memoire */
#define MextNoSync   	0	/* pas de synchro */
#define MextInstSync 	1	/* synchro a l'installation de l'acq */
#define MextStartSync	2	/* synchro au start de l'acq */
#define MextStopSync 	4	/* synchro au stop de l'acq */ 
#define MextExitSync	8	/* synchro a l'exit de l'acq */


/* Definition des protocoles de dialogue entre VAX et Serveur de buffers */
#define OldSbufProtocol    0  /* Ancien protocole SBUF            */
#define AsyncSbufProtocol  1  /* Protocole de dialogue asynchrone */
#define BurstSbufProtocol  2  /* Protocole avec envoi en rafale   */

/* Description des differents types de : 
				      _	controleurs VME CAENET (V288 ou A200)
 				      _ chassis H.T C.A.E.N sy127 ou sy403
				      _ echelles VME V260
*/

/* Chassis H.T	sur le CAENET */

#define	SY403	  1			/* Chassis H.T type SY403 	  */
#define SY127	  2			/* Chassis H.T type SY127 	  */
#define MAX_BOARD 4			/* nbre max de cartes par chassis */

/* Contoleurs VME CAENET */

#define A200	 1	/* Controleur CAENET low speed (48kBauds) */
#define V288	 2	/* Controleur CAENET high speed (1Mbauds) */
#define ID_V288  1	/* Ident du controleur CAENET V288        */
#define INTFLAG  1	/* Flag fonctionnement sous IT		  */

/* Echelles */

#define V260	1	/* Echelle C.A.E.N  16 voies 24 bits */


  /* Definition of Acquisition processes types */
#define Proc_DataInput          1  /* Data Readout */ 
#define Proc_SubEventMerge      2  /* Sub-event merge */
#define Proc_DataProcess        4  /* Data Processing */
#define Proc_DataOutput         8  /* Data Transfer to the host */
#define Proc_GenServ          0x10 /* General server */

  /* Definition of the maximum numbers of interfaces per readout process */
#define INTERFACE_MAX_NB 4

  typedef struct READOUTDES {     /* Readout process descriptor */
    UNSINT32 InterfaceNb;      /* Number of interfaces */
    char InterfaceName[INTERFACE_MAX_NB][EQUIP_NAME_LEN];
  } DATAINPUTDES;

  typedef struct MERGEDES {     /* Merge process descriptor */
    char InputBufstack[EQUIP_NAME_LEN]; /* Input bufstack name */
    int  BufstackFifo;          /* Fifo number for input bufstack */
    int  BufstackNb;            /* Number of input bufstacks */
  } MERGEDES;

  typedef struct DATAPROCDES {  /* Data processing process descriptor */
    char InputBufstack[EQUIP_NAME_LEN]; /* Input bufstack name */
    int  BufstackFifo;          /* Fifo number for input bufstack */
    int  BufstackNb;            /* Number of input bufstacks */
  } DATAPROCDES;

  typedef struct TRANSDES {     /* Data transfer process descriptor */
    char InputBufstack[EQUIP_NAME_LEN]; /* Input bufstack name */
    int  BufstackFifo;          /* Fifo number for input bufstack */
    int  BufstackNb;            /* Number of input bufstacks */
    UNSINT32 protocol;             /* Protocole de dialogue */
    char     IPname[EQUIP_NAME_LEN];   /* Nom Internet */
  } DATAOUTPUTDES;

  typedef struct PROCDES {     /* Acquisition process descriptor */
    char CpuName[EQUIP_NAME_LEN];  /* Cpu name executing the process */
    UNSINT32 CpuId;
    UNSINT32 ToSend;               /* Timeout send for message passing */
    UNSINT32 ToReceive;            /* Timeout receive for message passing */
    UNSINT32 DefOutputBufSize;     /* Default output buffer size */
    char ExeName[EXE_NAME_LEN];    /* Executable name */
    UNSINT32 ProcType;             /* Process type */
    union {
      DATAINPUTDES     Input;
      MERGEDES         Merge;
      DATAPROCDES      DataProc;
      DATAOUTPUTDES    Output;
    } type;
  } PROCDES;

typedef struct CPUDES {	/* descripteur de CPU . */
  char IPname[EQUIP_NAME_LEN];   /* Nom Internet */
  INT32    cputype;		/* type de cpu */
  INT32    appltyp;		/* type d'application */
  UNSINT32 xmaildev;		/* adresse du xmail vue du DEV */
  UNSINT32 xmailadr;		/* adresse du xmail vue du cible */
  UNSINT32 shmemvme;		/* adresse memoire partagee vue du VME */
  UNSINT32 shmemcpu;		/* adresse memoire partagee vue du cpu */
  UNSINT32 shmemsiz;	        /* taille memoire partagee */
  UNSINT32 am;                  /* am code VME pour acces a shmemvme */
  UNSINT32 shmemfree;           /* taille memoire non attribuee par MEMDEF */
  UNSINT32 shmemreladr;         /* Adresse relative du 1er octet libre */
  UNSINT32 CSRvmebase;          /* CSR space Vme address of the CPU 
				   used on CES CPU's */
  UNSINT32 CSRam;               /* CSR space AM code */
  } CPUDES;

typedef struct VMEMEMDES {	/* Descripteur de carte memoire VME */
  UNSINT32 vme_mem_adr;	        /* adresse physique VME */
  UNSINT32 mem_siz;	        /* taille en octets */
  UNSINT32 am;                  /* AM code */
  UNSINT32 shmemfree;           /* taille memoire non attribuee par MEMDEF */
  UNSINT32 shmemreladr;         /* Adresse relative du 1er octet libre */
  } VMEMEMDES;

typedef struct MEMDES {	   /* Descripteur de zone memoire */
  char     BoardName[EQUIP_NAME_LEN]; /* nom de la carte memoire associee */
  UNSINT32 BoardId;         /* Ident de la carte memoire associee */
  UNSINT32 type;            /* carte VME ou memoire CPU partagee */
  UNSINT32 siz;	            /* taille en octets */
  UNSINT32 rel_adr;         /* adresse relative */
  UNSINT32 am;              /* AM code si VME*/
  } MEMDES;


typedef struct VXIDES {    /* Descripteur d'un ensemble de chassis VXI */
  UNSINT32 FirstCrate;     /* Numero du premier chassis */
  UNSINT32 NbCrate;        /* Nombre de chassis */
  UNSINT32 Type;           /* Type d'acces au VXI (Local, VIC,...) */
  char     AccessBoard[EQUIP_NAME_LEN]; /* nom de l'interface d'acces au VXI
                                           "" si bus VME local */
  UNSINT32 AccessBoardType; /* Type de la carte VIC (VIC8250, VIC8251) */
  UNSINT32 AccessBoardAddr; /* Base adresse de la carte VIC */
  char     VxiCpuName[EQUIP_NAME_LEN]; /* nom du cpu associe au vxi */
  } VXIDES;


typedef struct VICDES {    /* Descripteur d'une carte VIC maitre */
  UNSINT32 Type;           /* Type de la carte : VIC8250, VIC8251... */
  UNSINT32 BranchNum;       /* Branch number to access VXI or CAMAC */
  char     VsbCpuName[EQUIP_NAME_LEN];     /* Nom Cpu connecte au bus VSB */
  UNSINT32 BaseAddr;       /* Base address de la carte */
  } VICDES;

typedef struct MDETDES {   /* Descripteur de coupleurs sur le VME avec les 
			   bus d'acquisition des differents multidetecteurs
			   (IFV pour FERA) */
  /*EQUIPDES equip;*/
  UNSINT32 md_adr;         /* adresse VME du coupleur */
  INT32    md_bufsz;	   /* taille des buffers */
  INT32    md_bufnb;	   /* nombre de buffers */
  INT32    free;
  } MDETDES;

typedef struct CAMDES {	   /* Descripteur d'un coupleur de branche CAMAC */
  /*EQUIPDES equip;*/
  UNSINT32 cam_adr;	   /* adresse de base de la carte coupleur CBD ou VIC */
  INT32    cam_type;	   /* type de coupleur (CBD_8210, CBD_8110, VCC_2117) */
  char     cam_cpu[EQUIP_NAME_LEN]; /* nom operationnel du cpu associe */
  UNSINT32 cam_first_branch; /* Numero 1ere branche geree par ce coupleur */
  UNSINT32 cam_nb_branch;  /* Nombre de branches gerees par ce coupleur */
  UNSINT32 cam_first_crate; /* Numero 1er chassis */
  UNSINT32 cam_nb_crate;    /* Nombre de chassis */
  char     AccessBoard[EQUIP_NAME_LEN]; /* nom de l'interface d'acces au VXI
					   si carte VIC */
  UNSINT32 AccessBoardType; /* Type de la carte VIC (VIC8250, VIC8251) */
} CAMDES;

typedef struct MEMEXTDES { /* Descripteur de zone memoire VME */
  /*EQUIPDES equip;*/
  UNSINT32 type;           /* carte VME ou CPU */
  UNSINT32 vme_mem_adr;	   /* adresse physique VME de la carte */
  UNSINT32 cpu_mem_adr;    /* adresse physique locale dans le CPU */
  UNSINT32 mem_siz;        /* taille en octets */
  UNSINT32 am;             /* AM code */
  UNSINT32 driver;         /* Driver de lecture :
			      - MextNoFifo : pas de FIFO, evt GANIL
			      - MextGanFifo: Fifo GANIL, evt GANIL 
			      - MextChimera: pas de FIFO, evt CHIMERA */
  char mbx_name[EQUIP_NAME_LEN];/* nom mailbox synchro avec coupleur externe */
  /* Definition des options de synchro par mailbox */
  UNSINT32  OptSync;  /* Options synchro : 
			 - MextNoSync   : pas de synchro (FALSE)
			 - MextInstSync : synchro a l'installation de l'acq
			 - MextStartSync: synchro au start de l'acq
			 - MextStopSync : synchro au stop de l'acq */
  } MEMEXTDES;

typedef struct AMBXDES {   /* Descripteur de mailbox acquisition */
  /*EQUIPDES equip;*/
  struct acq_mbx *mbx_adr; /* adr mailbox ( 0 si pas definie dans macro) */
  UNSINT32 mbx_it_adr;	   /* adr VME mot IT (0 si n'existe pas ) */
  INT32    mbx_it_siz;	   /* taille mot IT (8,16,32) */
  INT32    mbx_it_val;	   /* valeur a ecrire dans mot d'IT */
  INT32    mbx_to;         /* Timeout en ms to receive AMBX_ACK_EXEC */
  char free[12];
} AMBXDES;

typedef struct D2VBDES {  /* D2VB board descriptor */
  UNSINT32 reg_addr;      /* Register base address */
  UNSINT32 ram_addr;      /* Buffer RAM base address */
  UNSINT32 buf_siz;       /* Buffer size in bytes */
} D2VBDES;


/*--------------------------------------------------------------*/
/* Description des structures HTDES,CNETDES,ECHDES decrivant :	*/
/*								*/
/*		- Les hautes tensions INDRA (sy403 ou sy127)	*/
/* 		- Les modules echelles utilises (v260 )		*/
/*		- Les controleurs CAENET :			*/
/*					  - A200 pour sy127	*/
/*					  - V288 pour sy403     */
/*--------------------------------------------------------------*/

/* structure CNETDES */
 
typedef struct CNETDES {
  /*EQUIPDES equip;*/
  UNSINT32 base_addr;	          /* module base address   	*/ 
  INT32    type;		  /* type A200 ou V288		*/ 
  INT32    ident;	          /* ident du controleur	*/
  char     mycpu[EQUIP_NAME_LEN]; /* nom du cpu associe       	*/
  INT32    present;		  /* flag module present       	*/
  CPUDES *cpu;		          /* pointeur sur la struct cpu */
                                  /* associee a ce controleur   */
  } CNETDES;


/* structure des caracteristiques des cartes HT */
typedef struct VI_LIMITS {
  INT32 vmax[MAX_BOARD];
  INT32 imax[MAX_BOARD];
  INT32 resv[MAX_BOARD];
  INT32 resi[MAX_BOARD];
  INT32 decv[MAX_BOARD];
  INT32 deci[MAX_BOARD];
  } VI_LIMITS;

/* structure HTDES */
typedef struct HTDES {
  /*EQUIPDES equip;*/
  INT32    phy_crate;	            /* no chassis sur le caenet (1a 99)  */
  INT32    nb_mod;		    /* nbre de modules installes         */
				    /* sy403 : 4 modules de 16 voies     */
				    /* sy127 : 10 modules de 4 voies     */
  INT32    type;		    /* type sy127 ou sy403		 */
  char     mycntrl[EQUIP_NAME_LEN]; /* nom du controleur CAENET associe  */
  CNETDES *caenet;	            /* pointeur sur struct du controleur */
				    /* CAENET associe a ce chassis	 */
  INT32    present;		    /* flag de presence du chassis       */
  char     sy403_ident[12];         /* Ident du chassis & rev logiciel   */
  VI_LIMITS max_vi;                 /* caracteristiques des cartes HT    */
  } HTDES;


/* structure ECHDES */

typedef struct ECHDES {
  /*EQUIPDES equip;*/
  UNSINT32 base_adr;		    /* adresse de base du module 	*/
  INT32    type;		    /* type d'echelle (LECROY ou CAEN)  */
  char     mycpu[EQUIP_NAME_LEN];   /* nom cpu associe au module ech    */
  CPUDES  *cpu;		            /* pointeur su la cpu associee      */
  INT32    present; 	            /* flag presence du module	        */
  } ECHDES;

union Desc
{
    CPUDES    Cpu;
    VMEMEMDES VmeMem;
    MEMDES    Mem;
    VXIDES    Vxi;
    VICDES    Vic;
    MDETDES   Mdet;
    CAMDES    Cam;
    MEMEXTDES MemExt;
    AMBXDES   Ambx;
    CNETDES   Cnet;
    HTDES     Ht;
    ECHDES    Ech;
    PROCDES   Proc;
    D2VBDES   D2vb;
};

typedef struct EQUIPDES {   /* entete de tous les descripteurs d'equipement */
  char     name[EQUIP_NAME_LEN]; /* nom de l'equipement */
  INT32    NumEntry;        /* Numero d'entree dans la table */
  INT32    id;              /* ident unique pour l'ensemble des equipements */
  INT32    famille;         /* famille d'equipement */
  INT32    num;	            /* numero d'equipement dans la famille */
  INT32    GlobalName;      /* -1       si nom equipement global au chassis 
			       NumEntry du cpu associe si local a un cpu */
  union Desc des;
} EQUIPDES;                

typedef struct EQUIPHD {   /* entete de la table des equipements */
  char  AreaName[EQUIP_NAME_LEN]; /* Nom de la table permettant de verifier sa validite */
  UNSINT32 EquipSize;	   /* taille de la table en octets */
  INT32 EquipNbMax;        /* Nombre max d'equipements dans la table */
  INT32 EquipNb;           /* Nombre d'equipements dans la table 
			      en fait le numero de la 1ere entree a partir
			      de laquelle il n'y a plus d'equipements */
  INT32 EquipVersion;      /* Numero de version */
  sem_t EquipSem;          /* Semaphore d'acces a memoire des equipements */
  EQUIPDES D_ EquipTable;    /* adr relative table equipements de type EQUIPDES*/
  } EQUIPHD;


/* Prototypes generaux */

#ifndef __EquipPos_c
extern
#endif
int EquipPos    (int famille, char *equipnam, EQUIPDES **Equip);

#ifndef __EquipSui_c
extern
#endif
int EquipSui    (int famille, EQUIPDES **EquipDesCur);

#ifndef __EquipMemMap_c
extern
#endif
int EquipMemMap   (char *MemName, void **MemAdr, int OptMap);

#ifndef __EquipMemMap_c
extern
#endif
int EquipMemMapDes   (EQUIPDES *EquipDes, void **MemAdr, int OptMap);

#ifndef __EquipMemMap_c
extern
#endif
int EquipMemUnmap (char *MemName);

#ifndef __EquipMemMap_c
extern
#endif
int EquipMemUnmapDes (EQUIPDES *EquipDes);

#ifndef __EquipShm_c
extern
#endif
int EquipShmCreate(char *MemName, int MemSize, 
		   int AccesMode, char **MemLogAdr);

#ifndef __EquipShm_c
extern
#endif
int EquipShmDelete(char *MemName);

#ifndef __EquipGetMyCpuName_c
extern
#endif
int EquipGetMyCpuName(char **CpuName);


#ifdef __cplusplus
}
#endif


#endif
