/**************************************************************************
   EquipVic.h

   Mapping du bus VMV depuis les bus VME et VSB, via une carte CES VIC8250

   Auteur : Frederic Saillant

   Date de derniere mise a jour : 22 janvier 1999

   Modifications :
                   - 19 janvier 1999  Creation
                   - 22 janvier 1999  AM codes

***************************************************************************/

#ifndef _EQUIPVIC_H_
#define _EQUIPVIC_H_


/*****************************************************************/

/* Acces a des chassis CAMAC controles par une carte CES VCC2117 */

   /* Numeros min et max pour un chassis CAMAC sur le bus VMV */

#define VMV_MIN_CAMAC_CRATE_NUM        1
#define VMV_MAX_CAMAC_CRATE_NUM        14

   /* Acces depuis le bus VME en A24 */

#define VMV_CAMAC_VME_AM             0x39

#define VMV_CAMAC_VME_BASE           0

#define VMV_CAMAC_VME_INCREMENT      0x00100000

   /* VME_address = VMV_CAMAC_VME_BASE + (CrateNum * VMV_CAMAC_VME_INCREMENT) */

   /**********************************/

   /* Acces depuis le bus VSB */

#define VMV_CAMAC_VSB_BASE           0xFF000000

#define VMV_CAMAC_VSB_INCREMENT      0x00100000

   /* VSB_address = VMV_CAMAC_VSB_BASE + (CrateNum * VMV_CAMAC_VSB_INCREMENT) */

   /**********************************/

   /* Formules de calcul des numeros de pages VME A24 de la carte VIC8250 */

#define VMV_CAMAC_VME_PAGE_BASE      0x1FF0 + (VMV_CAMAC_VME_BASE >> 20)

#define VMV_CAMAC_VME_PAGE_INCREMENT (VMV_CAMAC_VME_INCREMENT >> 20)

   /* page_num = VMV_CAMAC_VME_PAGE_BASE + (CrateNum * VMV_CAMAC_VME_PAGE_INCREMENT) */

   /**********************************/

   /* Formules de calcul des numeros de pages VSB de la carte VIC8250 */

#define VMV_CAMAC_VSB_PAGE_BASE      (VMV_CAMAC_VSB_BASE >> 20)

#define VMV_CAMAC_VSB_PAGE_INCREMENT (VMV_CAMAC_VSB_INCREMENT >> 20)

   /* page_num = VMV_CAMAC_VSB_PAGE_BASE + (CrateNum * VMV_CAMAC_VSB_PAGE_INCREMENT) */

/***************************************************************/


/* Acces a des chassis VXI controles par une carte CES VIC8250 */

   /* Numeros min et max pour un chassis VXI sur le bus VMV */

#define VMV_MIN_VXI_CRATE_NUM        1
#define VMV_MAX_VXI_CRATE_NUM        12

   /**********************************/

   /* Acces depuis le bus VME en A32 */

#define VMV_VME_AM                   0x09

#define VMV_VME_BASE                 0x10000000

#define VMV_A16_VME_OFFSET           0x00000000

#define VMV_A16_VME_BASE             (VMV_VME_BASE + VMV_A16_VME_OFFSET)

#define VMV_A16_VME_INCREMENT        0x00100000

#define VMV_A24_VME_OFFSET           0x0D000000

#define VMV_A24_VME_BASE             (VMV_VME_BASE + VMV_A24_VME_OFFSET)

#define VMV_A24_VME_INCREMENT        0x00100000

#define VMV_A32_VME_OFFSET           0x00000000

#define VMV_A32_VME_BASE             (VMV_VME_BASE + VMV_A32_VME_OFFSET)

#define VMV_A32_VME_INCREMENT        0x01000000

   /**********************************/

   /* Acces depuis le bus VSB */

#define VMV_VSB_BASE                 0xF0000000

#define VMV_A16_VSB_OFFSET           0x00000000

#define VMV_A16_VSB_BASE             (VMV_VSB_BASE + VMV_A16_VSB_OFFSET)

#define VMV_A16_VSB_INCREMENT        0x00100000

#define VMV_A24_VSB_OFFSET           0x0D000000

#define VMV_A24_VSB_BASE             (VMV_VSB_BASE + VMV_A24_VSB_OFFSET)

#define VMV_A24_VSB_INCREMENT        0x00100000

#define VMV_A32_VSB_OFFSET           0x00000000

#define VMV_A32_VSB_BASE             (VMV_VSB_BASE + VMV_A32_VSB_OFFSET)

#define VMV_A32_VSB_INCREMENT        0x01000000

   /************************************/

   /* Formules de calcul des numeros de pages VME A32 de la carte VIC8250 */

#define VMV_A16_VME_PAGE_BASE        (0x1000 + (VMV_A16_VME_BASE >> 20))

#define VMV_A16_VME_PAGE_INCREMENT   (VMV_A16_VME_INCREMENT >> 20)

#define VMV_A24_VME_PAGE_BASE        (0x1000 + (VMV_A24_VME_BASE >> 20))

#define VMV_A24_VME_PAGE_INCREMENT   (VMV_A24_VME_INCREMENT >> 20)

#define VMV_A32_VME_PAGE_BASE        (0x1000 + (VMV_A32_VME_BASE >> 20))

#define VMV_A32_VME_PAGE_INCREMENT   (VMV_A32_VME_INCREMENT >> 20)

   /************************************/

   /* Formules de calcul des numeros de pages VSB de la carte VIC8250 */

#define VMV_A16_VSB_PAGE_BASE        (VMV_A16_VSB_BASE >> 20)

#define VMV_A16_VSB_PAGE_INCREMENT   (VMV_A16_VSB_INCREMENT >> 20)

#define VMV_A24_VSB_PAGE_BASE        (VMV_A24_VSB_BASE >> 20)

#define VMV_A24_VSB_PAGE_INCREMENT   (VMV_A24_VSB_INCREMENT >> 20)

#define VMV_A32_VSB_PAGE_BASE        (VMV_A32_VSB_BASE >> 20)

#define VMV_A32_VSB_PAGE_INCREMENT   (VMV_A32_VSB_INCREMENT >> 20)

/******************************************************************/

#endif
