#
# serv.tcl
#
# This file contains some utilities for talking with the ACS, SURV & D2VB
# It operates as a task
# It contains hooks for components which will probably be in the same crate
# i.e. ACS & SURV
#
# Simon Letts 17 may 1999
#
# Modifications :
#     B. Raine 06 august 2001
#        MIDASlogger name set in ACS iniacq command by using MIDASlogger 
#        environment variable if it exists

# FOR TESTING - scl 25/05/99
# set env(ServServer) ACS
set env(D2VBServer) VME

if {[info exists env(ServServer)]} then {set ServServer $env(ServServer)} \
    else {set ServServer VME}

if {[info exists env(D2VBServer)]} then {set D2VBServer $env(D2VBServer)} \
    else {set D2VBServer $ServServer}

if {[info exists env(MIDASlogger)]} then {set MIDASlogger $env(MIDASlogger)} \
	else {set MIDASlogger gansex1}

set debug 0
set EGreport 0
set started 0

set frame SERV
set title "Acquisition Control & Status Servers"
set icontitle AcqServ

set ServLog 1
set ServDebug 0
set ServCheck 1
set ServCode 0
set ServReply ""
set ACSDebug 0
set SURVDebug 0

set Serveurs(0) "EXOGAMD2VB"
set Serveurs(0) "ExogamD2VB"
set Serveurs(1) "Merge"
set Serveurs(2) "Processing"
set Serveurs(3) "OutPut"
set Serveurs(4) "ACS"
set Serveurs(5) "SURV"

set D2VBserv  SERV.$Serveurs(0)
set ACSserv   SERV.$Serveurs(4)
set SURVserv  SERV.$Serveurs(5)

set D2VBcommands(0)	"stop"
set D2VBcommands(1)	"start"
set D2VBcommands(2)	"init"
set D2VBcommands(3)	"stats_clear"
# Use the next one with care !!
set D2VBcommands(4)	"exit"

set ACScommands(0)	"iniacq"
set ACScommands(1)	"startacq"
set ACScommands(2)	"stopacq"
set ACScommands(3)	"exitacq"
set ACScommands(4)	"killacq"
set ACScommands(5)	"init"
set ACScommands(6)	"start"
set ACScommands(7)	"stop"
set ACScommands(8)	"set"
set ACScommands(9)	"exit"
set ACScommands(10)	"kill"

set SEToptions(0)	"debug"
set SEToptions(1)	"transmit"
set SEToptions(2)	"rawdata"
set SEToptions(3)	"checksum"
set SEToptions(4)	"histogram"
set SEToptions(5)	"event headers"
set SEToptions(6)	"D2VB CRC"

set selopts(0) 0
set selopts(1) 0
set selopts(2) 0
set selopts(3) 0
set selopts(4) 0
set selopts(5) 0
set selopts(6) 0

set SURVcommands(0)	"GET_INFO"
set SURVcommands(1)	"GET_STATUS"
set SURVcommands(2)	"GET_COUNTERS"
set SURVcommands(3)	"GET_ACQGLOB"
set SURVcommands(4)	"EXIT"

set astates {S_IDLE S_INITING S_INIT S_STARTING S_RUN S_STOPPING S_EXITING S_KILLING} 
set astatestrings {"Idle" "Initialising" "Initialised"  "Starting" "Running" "Stopping" "Exiting" "Killing"} 

set pstates {S_LOADING S_LOAD S_INITING S_INIT S_RUN S_STOPPING S_EXIT S_KILL} 
set pstatestrings {"Loading" "Loaded" "Initialising" "Initialised" "Running" "Stopping" "Exited" "Killed"} 

set statuses {F_ERR F_ERR_INIT F_ERR_RUN F_ERR_STOP F_ERR_TRANSMIT F_W_OUT_BUF F_DEAD } 
set statusstrings {"Error" "Initialise Error" "Run Error"  "Stop Error" "Network Transmission Error" "Waiting for Output Buffer" "Dead"} 

#set types {NONE INPUT MERGE DATA_PROC OUTPUT SERV} 
set types {NONE DataInput DataMerge DataProcessing DataOutput SERV}
set typestrings {"No Process Name" "Data Input" "Merge" "Data Processing" "Data Output"  "General Server"} 

set options {O_TRANSMIT O_DEBUG O_RAW O_CHECK O_HISTO O_EVENTHD O_D2VBCRC} 
set optstrings [list $SEToptions(1) $SEToptions(0) $SEToptions(2) $SEToptions(3) $SEToptions(4) $SEToptions(5) $SEToptions(6)] 

set alives {ALIVE DEAD NONE} 
set alivestrings {"Alive" "Dead" "Don`t care"} 

set defname	"ExogamTest"
set defbufsize	0x4000
set defproc	""

set expname	$defname
set bufsize	$defbufsize
set curproc	$defproc

proc ersdefine {server register class} {
   EG "register :name '/$server/$register' :class '$class' !new"
}

proc erswrite { server register args } {
   if {$args == "{}"} then {set args ""}
   EG "register :name '/$server/$register' :value '[eval lreplace $args 0 -1]'"
}

proc ersinquire {server register} {
   EG "register :pattern '/$server/$register' ?names"
}

proc erswta { server register args } {
   if {$args == "{}"} then {set args ""}
   EG "register :name '/$server/$register' :object '[eval lreplace $args 0 -1]'"
}

proc ersrda {server register} {
   EG "register  :name '/$server/$register' ?object "
}

proc ersread { server register } {
   EG "register :name '/$server/$register' ?value "
}

proc write-Serveur {S s args} {
   global ServDebug

   if {$args == "{}"} then {set args ""}
   erswrite $S $s [eval lreplace $args 0 -1]
}

proc read-Serveur {S s args} {
   global ServDebug

   erswta $S $s [eval lreplace " -rserver" 0 -1 [eval lreplace $args 0 -1 "-c "]]
   ersread $S $s
}

proc read-Serveur-status {S s} {
   global ServDebug

   erswta $S $s -rcode
   ersread $S $s
}

proc read-Serveur-reply {S s} {
   global ServDebug

   erswta $S $s -rreply
   ersread $S $s
}

proc write-D2VB-command-name {c} {
   global ServDebug ServCheck ServCode ServReply
   global D2VBServer D2VBserv

   set s ""

   write-Serveur $D2VBServer $D2VBserv $c
   if {$ServCheck} then {
      set ServCode [read-Serveur-status $D2VBServer $D2VBserv]; set s $ServCode
      set ServReply [read-Serveur-reply $D2VBServer $D2VBserv]
      if {$ServCode == "COD_WARNING"} then {
         insert-log "Warning - $ServReply"
      }

      if {$ServDebug} then {
         insert-debug "D2VB Write status: $ServCode"
         insert-debug "D2VB Write response: $ServReply"
      }

      if {$ServCode == "COD_ERROR"} then {
        midas-warning "D2VB Serveur error received: \"$ServCode - $ServReply\""
        return
      }
      if {$ServCode == -1} then {
        midas-warning "Could not contact D2VB Process Serveur"
        return
      }
   }
   return $s
}

proc write-D2VB-command {n} {
   global ServDebug ServCheck ServCode ServReply
   global D2VBServer D2VBserv D2VBcommands

   set s ""
   write-Serveur $D2VBServer $D2VBserv $D2VBcommands($n)
   if {$ServCheck} then {
      set ServCode [read-Serveur-status $D2VBServer $D2VBserv]; set s $ServCode
      set ServReply [read-Serveur-reply $D2VBServer $D2VBserv]
      if {$ServCode == "COD_WARNING"} then {
         insert-log "Warning - $ServReply"
      }

      if {$ServDebug} then {
         insert-debug "D2VB Write status: $ServCode"
         insert-debug "D2VB Write response: $ServReply"
      }

      if {$ServCode == "COD_ERROR"} then {
        midas-warning "D2VB Serveur error received: \"$ServCode - $ServReply\""
        return
      }
      if {$ServCode == -1} then {
        midas-warning "Could not contact D2VB Process Serveur"
        return
      }
   }
   return $s
}

proc read-D2VB-command {n} {
   global ServDebug ServCheck ServCode ServReply
   global D2VBServer D2VBserv D2VBcommands

   set ServReply [read-Serveur $D2VBServer $D2VBserv $D2VBcommands($n)]
   if {$ServCheck} then {
      set ServCode [read-Serveur-status $D2VBServer $D2VBserv]
      if {$ServCode == "COD_WARNING"} then {
         insert-log "Warning - $ServReply"
      }

      if {$ServDebug} then {
         insert-debug "D2VB Read status: $ServCode"
         insert-debug "D2VB Read response: $ServReply"
      }

      if {$ServCode == "COD_ERROR"} then {
        midas-warning "D2VB Serveur error received: \"$ServCode - $ServReply\""
        return
      }
      if {$ServCode == -1} then {
        midas-warning "Could not contact D2VB Process Serveur"
        return
      }
   }
   return $ServReply
}

proc write-ACS-command {pr co args} {
   global ServDebug ServServer ServCheck ServCode ServReply ACSserv

   if {$args == "{}"} then {set args ""}

   write-Serveur $ServServer $ACSserv [eval lreplace {$args} 0 -1 $co $pr]

   set s ""
   if {$ServCheck} then {
      set ServCode [read-Serveur-status $ServServer $ACSserv]; set s $ServCode
      set ServReply [read-Serveur-reply $ServServer $ACSserv]
      if {$ServCode == "COD_WARNING"} then {
         insert-log "Warning - $ServReply"
      }

      if {$ServDebug} then {
         insert-debug "ACS Write status: $ServCode"
         insert-debug "ACS Write response: $ServReply"
      }

      if {$ServCode == "COD_ERROR"} then {
        midas-warning "ACS Serveur error received: \"$ServCode - $ServReply\""
        return
      }
      if {$ServCode == -1} then {
        midas-warning "Could not contact ACS Serveur"
        return
      }
   }
   return $s
}

proc read-ACS-command {pr co args} {
   global ServDebug ServServer ServCheck ServCode ServReply ACSserv

   if {$args == "{}"} then {set args ""}
   set ServReply [read-Serveur $ServServer $ACSserv [eval lreplace {$args} 0 -1 $co $pr]]

   if {$ServCheck} then {
      set ServCode [read-Serveur-status $ServServer $ACSserv]
      if {$ServCode == "COD_WARNING"} then {
         insert-log "Warning - $ServReply"
      }

      if {$ServDebug} then {
         insert-debug "ACS Read status: $ServCode"
         insert-debug "ACS Read response: $ServReply"
      }

      if {$ServCode == "COD_ERROR"} then {
        midas-warning "ACS Serveur error received: \"$ServCode - $ServReply\""
        return
      }
      if {$ServCode == -1} then {
        midas-warning "Could not contact ACS Serveur"
        return
      }
   }
   return $ServReply
}

proc inquire-SURV-proc {n args} {
   global ServDebug ServServer ServCheck ServCode ServReply
   global SURVserv SURVcommands

   if {$args == "{}"} then {set args ""}
   set ServReply [read-Serveur $ServServer $SURVserv [eval lreplace {$args} 0 -1 $SURVcommands($n)]]

   if {$ServCheck} then {
      set ServCode [read-Serveur-status $ServServer $SURVserv]
      if {$ServCode == "COD_WARNING"} then {
         insert-log "Warning - $ServReply"
      }
      if {$ServDebug} then {
         insert-debug "SURV Read status: $ServCode"
         insert-debug "SURV Read response: $ServReply"
      }

      if {$ServCode == "COD_ERROR"} then {
        midas-warning "SURVEYOR Serveur error received: \"$ServCode - $ServReply\""
        return
      }
      if {$ServCode == -1} then {
        midas-warning "Could not contact SURVEYOR Serveur"
        return
      }
   }
   return $ServReply
}

proc get-SURV-info args {
   inquire-SURV-proc 0 $args
}

proc get-SURV-counters args {
   inquire-SURV-proc 2 $args
}

proc inquire-SURV n {
   global ServDebug ServServer ServCheck ServCode ServReply
   global SURVserv SURVcommands

   set ServReply [read-Serveur $ServServer $SURVserv $SURVcommands($n)]

   if {$ServCheck} then {
      set ServCode [read-Serveur-status $ServServer $SURVserv]
      if {$ServCode == "COD_WARNING"} then {
         insert-log "Warning - $ServReply"
      }

      if {$ServDebug} then {
         insert-debug "SURV Read status: $ServCode"
         insert-debug "SURV Read response: $ServReply"
      }

      if {$ServCode == "COD_ERROR"} then {
        midas-warning "SURVEYOR Serveur error received: \"$ServCode - $ServReply\""
        return
      }
      if {$ServCode == -1} then {
        midas-warning "Could not contact SURVEYOR Serveur"
        return
      }
   }
   return $ServReply
}

proc get-SURV-status {} {
   inquire-SURV 1
}

proc get-SURV-globmem {} {
   inquire-SURV 3
}

proc stop-D2VB {} {
   write-D2VB-command 0
}

proc start-D2VB {} {
   write-D2VB-command 1
}

proc init-D2VB {} {
   write-D2VB-command 2
}

proc clear-D2VB-stats {} {
   write-D2VB-command 3
}

proc exit-D2VB-too {} {
   write-D2VB-command 4
}

proc exit-D2VB {} {
   global D2VBServer D2VBserv
   exit-Serveur $D2VBServer $D2VBserv
}

proc exit-SURV {} {
   global ServServer SURVserv
   exit-Serveur $ServServer $SURVserv
}

proc exit-ACS {} {
   global ServServer ACSserv
   exit-Serveur $ServServer $ACSserv
}

proc exit-Serveur {Server serv} {
   write-Serveur $Server $serv exit
}

proc do-ACS-command {pr n args} {
   global ACScommands

   set a ""
   for {set i 0} {$i<[llength $args]} {incr i} {
      set a [lappend a [lindex $args $i]]
   }
   write-ACS-command $pr $ACScommands($n) $a
}

proc click-sbufsize v {
   global defbufsize bufsize
   set bufsize $defbufsize
   :modify SBUFSIZE :value $defbufsize
}

proc click-sexpname v {
   global expname
   set expname $v
   if {[winfo ismapped .]} then click-redisplay-sexpname
}

proc do-iniacq {BufSz ExpName} {
   global ServDebug ServServer ServCheck ServCode ServReply
   global ACScommands ACSserv
   global MIDASlogger

   erswrite $ServServer $ACSserv "$ACScommands(0) $BufSz $ExpName $MIDASlogger"
   if {$ServCheck} then {
      set ServCode [read-Serveur-status $ServServer $ACSserv]; set s $ServCode
      set ServReply [read-Serveur-reply $ServServer $ACSserv]
      if {$ServCode == "COD_WARNING"} then {
         insert-log "Warning - $ServReply"
      }

      if {$ServDebug} then {
         insert-debug "do-iniacq ACS Write status: $ServCode"
         insert-debug "do-iniacq ACS Write response: $ServReply"
      }

      if {$ServCode == "COD_ERROR"} then {
        midas-warning "ACS Serveur error received: \"$ServCode - $ServReply\""
        return
      }
      if {$ServCode == -1} then {
        midas-warning "Could not contact ACS Serveur"
        return
      }
   }
}

proc do-initacq {} {
   global bufsize expname
   do-iniacq $bufsize $expname
}

proc click-sinitacq {} {
   do-initacq
   click-sseget
}

proc click-sstartacq {} {
   global ACScommands
   write-ACS-command "" $ACScommands(1)
   click-sseget
}

proc click-sstopacq {} {
   global ACScommands
   write-ACS-command "" $ACScommands(2)
   click-sseget
}

proc click-sexitacq {} {
   global ACScommands
   write-ACS-command "" $ACScommands(3)
   click-sseget
}

proc click-skillacq {} {
   global ACScommands
   write-ACS-command "" $ACScommands(4)
   click-sseget
}

proc click-sprocname v {
   global curproc Serveurs
   set curproc $v
   if {$curproc != $Serveurs(0)} then {
      :modify SSELOPT2 :inactive T

      :modify SSELOPT4 :inactive T

      :modify SSELOPT5 :inactive T

      :modify SSELOPT6 :inactive T
   } else {
      :modify SSELOPT2 :inactive F

      :modify SSELOPT4 :inactive F

      :modify SSELOPT5 :inactive F

      :modify SSELOPT6 :inactive F
   }
   click-sspget
}

proc click-sprocclear {} {
   click-sprocname ""
   :modify SPROCNAME :value ""
}

proc click-sprocall {} {
   click-sprocname "ALL"
   :modify SPROCNAME :value "ALL"
}

proc click-sprocd2vb {} {
   global Serveurs
   click-sprocname $Serveurs(0)
   :modify SPROCNAME :value $Serveurs(0)
}

proc click-sprocoutput {} {
   global Serveurs
   click-sprocname $Serveurs(3)
   :modify SPROCNAME :value $Serveurs(3)
}

proc do-initpr {pr} {
   global ACScommands
   write-ACS-command $pr $ACScommands(5)
}

proc click-sinitproc {} {
   global curproc
   do-initpr $curproc
   click-sspget
}

proc do-startpr {pr} {
   global ACScommands
   write-ACS-command $pr $ACScommands(6)
}

proc click-sstartproc {} {
   global curproc
   do-startpr $curproc
   click-sspget
}

proc do-stoppr {pr} {
   global ACScommands
   write-ACS-command $pr $ACScommands(7)
}

proc click-sstopproc {} {
   global curproc
   do-stoppr $curproc
   click-sspget
}

proc do-exitpr {pr} {
   global ACScommands
   write-ACS-command $pr $ACScommands(9)
}

proc click-sexitproc {} {
   global curproc
   do-exitpr $curproc
   click-sspget
}

proc do-killpr {pr} {
   global ACScommands
   write-ACS-command $pr $ACScommands(10)
}

proc click-skillproc {} {
   global curproc
   do-killpr $curproc
   click-sspget
}

proc click-selopt args {
   global selopts
   set n [first $args]
   set selopts($n) [expr $selopts($n) + 1 & 1]
}

proc do-setpr {pr opt} {
   global ACScommands
   catch {write-ACS-command $pr $ACScommands(8) $opt}
}

proc click-setopt {} {
   global curproc SEToptions selopts
   for {set i 0} {$i < [array size selopts]} {incr i} {
      if {$selopts($i)} then {do-setpr $curproc $SEToptions($i)}
   }
   click-sspget
}

proc do-unsetpr {pr opt} {
   global ACScommands
   catch {write-ACS-command $pr $ACScommands(8) no$opt}
}

proc click-clropt {} {
   global curproc SEToptions selopts
   for {set i 0} {$i < [array size selopts]} {incr i} {
      if {$selopts($i)} then {do-unsetpr $curproc $SEToptions($i)}
   }
   click-sspget
}

proc update-exp s {
   global statuses
   set l [split $s ,]
   :modify SSEXPNAME :value [first $l]
   :modify SSSTATE :value [astate2string [string trimleft [second $l]]]
   :modify SSCOMMENT :value [first [last $l]]
   :modify SSSTATUS :value [chlst2ch $statuses [lreplace [lreplace $l 0 1] end end]]
}

proc click-sseget {} {
   global ServLog
   set s [get-SURV-status]
   if {$ServLog} then {
      insert-log "Experiment Server Status at [date]:"
      insert-log "$s"
   }
   update-exp $s
}

proc click-sglobmem {} {
   global ServLog
   set m [get-SURV-globmem]
   if {$ServLog} then {
      insert-log "Experiment Global Memory at [date]:"
      insert-log "$m"
   }
   midas-report $m
}

proc update-proc s {
   global alives statuses options
   set l [split [first $s] ,]

   set opts {}
   set stat {}

   set statopt [lreplace $l 0 3]
   for {set i 0} {$i < [llength $statopt]} {incr i} {
      set so [string trimleft [lindex $statopt $i]]
      if {[string first "F_" $so] != -1} then {
         set stat [lappend stat $so]
      } else {
         if {[string first "O_" $so] != -1} then {
            set opts [lappend opts $so]
         }
      }
   }
   :modify SSPROCNAME :value [string trimleft [first $l]]
   :modify SSPTYPE :value [type2string [string trimleft [second $l]]]
   :modify SSPALIVE :value [lsearch $alives [string trimleft [third $l]]]
   :modify SSPSTATE :value [pstate2string [string trimleft [fourth $l]]]
   :modify SSPSTATUS :value [chlst2ch $statuses $stat]
   :modify SSPOPTIONS :value [chlst2ch $options $opts]
}

proc click-sspget {} {
   global curproc ServLog
   set s [get-SURV-info $curproc]
#set s "test23, S_KILLING, F_ERR_INIT,F_ERR_STOP,\"Ho there\"\n Merge, MERGE, NONE, S_EXIT, F_W_OUT_BUF, O_DEBUG\n"
#set s "test23, S_KILLING, F_ERR_INIT,F_ERR_STOP,\"Ho there\"\n Merge, MERGE, NONE, S_EXIT, F_ERR_INIT,O_RAW,O_TRANSMIT\n D2VB, DATA_PROC, DEAD, S_KILL, O_DEBUG,O_TRANSMIT\n"
# set s "Merge, MERGE, DEAD, S_RUN, F_ERR_INIT,O_DEBUG"
   set ll [split $s `\n`]
   if {$curproc==""} then {
      update-exp [first $ll]
      set ll [lreplace $ll 0 0]
   }
   if {$ServLog} then {
      insert-log "Process Info at [date]:"
      insert-log "$s"
   }
   update-proc  $ll
}

proc is-transmitting pr {
   global ServLog
   if {$pr == "" || $pr == "ALL"} then {return}
   set s [get-SURV-info $pr]
   set ll [split $s `\n`]
   if {$ServLog} then {
      insert-log "Process Info at [date]:"
      insert-log "$s"
   }
   set l [split [first $ll] ,]
   set statopt [lreplace $l 0 3]
   set tr [lsearch $statopt "O_TRANSMIT"]
   if {$tr == -1} then {
      return 0
   } else {
      return 1
   }
}


proc click-redisplay-sexpname {} {
   global expname
   :modify SEXPNAME :value $expname
}

proc click-redisplay {} {
   click-redisplay-sexpname
   click-sseget
   click-sspget
}

proc get-ACS-state long {
   global astates
# if long is false just send 0/1
# 1 is S_RUN, 0 is any other state
   set v [lsearch $astates [string trimleft [second [split [get-SURV-status] ,]]]]
   if {$long} then {
      return $v
   }
   if {$v==4} then {set v 1} else {set v 0}
   return $v
}

proc get-D2VB-state {} {
   global Serveurs
   get-SERVEUR-state $Serveurs(0)
}

proc get-OUTPUT-state {} {
   global Serveurs
   get-SERVEUR-state $Serveurs(3)
}

# set s "ExogamD2VB, DATA_PROC, NONE, S_RUN, F_W_OUT_BUF, O_DEBUG"
#   set s [get-SURV-info $sv]
#   set l [split $s ,]
#   set s [string trimleft [fourth $l]]
#   set v [lsearch $pstates $s]
#   lsearch $pstates [string trimleft [fourth [split [get-SURV-info $sv] ,]]]

proc get-SERVEUR-state sv {
   global pstates
   lsearch $pstates [string trimleft [fourth [split [get-SURV-info $sv] ,]]]
}

proc code2string {code codelist strings} {
   lindex $strings [lsearch $codelist $code]
}

proc astate2string state {
   global astates astatestrings
   code2string $state $astates $astatestrings
}

proc pstate2string state {
   global pstates pstatestrings
   code2string $state $pstates $pstatestrings
}

proc status2string  status {
   global statuses statusstrings
   code2string $status $statuses $statusstrings
}

proc opt2string opt {
   global options optstrings
   code2string $opt $options $optstrings
}

proc type2string type {
   global types typestrings
   code2string $type $types $typestrings
}

proc chlst2ch  {chlist st} {
   set v 0
   foreach i $st {
      set v [expr $v + pow(2, [lsearch $chlist [string trimleft $i]])]
   }
   expr int($v)
}

proc make-frame {} {
   global defname defbufsize defproc SEToptions 
   global astatestrings pstatestrings alivestrings statusstrings optstrings

   :frame [self frame-name] :label "Acquisition Control & Status Servers" :icon-name "[self icon-title]" :show-footer T :resize F
   
   :panel PANEL1 :layout T :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 15
      
         :message MSG1 :label "Experiment" :bold T
         :next-row T
         :text SEXPNAME :label "Experiment Name"  :value $defname :w 20
         :button SGLOBMEM :label "Show Global Memory"

   :panel PANEL2 :layout T :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 25
      
         :message MSG2 :label "Control" :bold T
         :next-row T
         :button SINITACQ :label "Initialise Acquisition"
         :text SBUFSIZE :label "Buffer Size"  :value $defbufsize :w 8 :deaf T
      :next-row T
         :button SSTOPACQ :label "Stop Acquisition"
         :button SSTARTACQ :label "Start Acquisition"
      :next-row T
         :button SEXITACQ :label "Exit Acquisition Processes"
         :button SKILLACQ :label "Kill Acquisition Processes"

   :panel PANEL3 :layout T :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 15
         :message MSG3 :label "Status" :bold T
      :next-row T
         :text SSEXPNAME :label "Name"  :w 20 :deaf T
         :text SSSTATE :label "State" :w 15 :deaf T
         :button SSEGET :label "Update"
      :next-row T
         :choice SSSTATUS :label "Status" :exclusive F :strings $statusstrings :value 0 :deaf T
      :next-row T
         :text SSCOMMENT :label "Comment"  :w 40 :deaf T

   :panel PANEL4 :layout T :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 5
         :message MSG4 :label "Per Process" :bold T
         :next-row T
         :text SPROCNAME :label "Process Name"  :value $defproc :w 20
         :button SPROCCLEAR :label "Clear"
         :button SPROCD2VB :label "D2VB"
         :button SPROCOUTPUT :label "Output"
         :button SPROCALL :label "ALL"

   :panel PANEL5 :layout L :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 15
         :message MSG5 :label "Control" :bold T
         :next-row T; :row-gap 5
         :button SINITPROC :label "Initialise Process"
      :next-row T
         :button SSTOPPROC :label "Stop Process"
         :button SSTARTPROC :label "Start Process"
      :next-row T
         :button SEXITPROC :label "Exit Process"
         :button SKILLPROC :label "Kill Process"

   :panel PANEL6 :layout L :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 5
         :message MSG6 :label "Options" :bold T
         :next-row T; :row-gap 5

         :checkbox SSELOPT1 :label "$SEToptions(1)" :valuex 14c :command "click-selopt 1"

         :checkbox SSELOPT3 :label "$SEToptions(3)" :valuex 14c :command "click-selopt 3"
      :next-row T
         :checkbox SSELOPT0 :label "$SEToptions(0)" :valuex 14c :command "click-selopt 0"
      :next-row T
         :checkbox SSELOPT2 :label "$SEToptions(2)" :valuex 14c :command "click-selopt 2" :inactive T

         :checkbox SSELOPT4 :label "$SEToptions(4)" :valuex 14c :command "click-selopt 4" :inactive T
      :next-row T
         :checkbox SSELOPT5 :label "$SEToptions(5)" :valuex 14c :command "click-selopt 5" :inactive T

         :checkbox SSELOPT6 :label "$SEToptions(6)" :valuex 14c :command "click-selopt 6" :inactive T

      :next-row T
         :message SOPTMSG :label "Act on selected options:"
         :button SSETOPT :label "Set" :command "click-setopt"
         :button SCLROPT :label "Clear" :command "click-clropt"

   :panel PANEL7 :layout B :padx 5 :pady 5 :after FOOTER
      :layout H; :row-gap 10; :col-gap 15
         :message MSG7 :label "Status" :bold T
      :next-row T
         :text SSPROCNAME :label "Name:"  :w 20 :deaf T
         :text SSPSTATE :label "State:" :w 15 :deaf T
         :button SSPGET :label "Update"
      :next-row T
         :text SSPTYPE :label "Type:"  :w 16 :deaf T
         :choice SSPALIVE :label "Alive:" :exclusive T  :strings $alivestrings :value 0 :deaf T
      :next-row T
         :choice SSPSTATUS :label "Status:" :exclusive F :strings $statusstrings :value 0 :deaf T
      :next-row T
         :choice SSPOPTIONS :label "Options" :exclusive F :strings $optstrings :value 0 :deaf T

   :show T         
}

proc start {} {
   global started
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
#   click-redisplay
}
