
set debug 0
set EGreport 0
set started 0

set frame STATS
set title "Event Builder Statistics"
set icon EbStats

set d2vbstats "D2VB.Stats"
set stat_lines 17

# variables in the EB's D2VB statistics record
set dratedef(0) "blocks input"
set dratedef(1) "blocks output"
set dratedef(2) "events input"
set dratedef(3) "events output"

set dratedef(4) "no start token"
set dratedef(5) "unexpected data"
set dratedef(6) "last event count"
set dratedef(7) "event length error"

set dratedef(8) "vre endsubevent error"
set dratedef(9) "subevent length error"
set dratedef(10) "subevent eventnumber error"
set dratedef(11) "subevent eventnumber missing"

set dratedef(12) "vre bad readout"
set dratedef(13) "unenabled data"
set dratedef(14) "wrong vre error"
set dratedef(15) "subevent crc error"

set dratedef(16) "vre re-sync"


set CurrentServer [inform serv inquire D2VBServer]

if {[info exists CurrentServer] == 0} then {set CurrentServer "vme1"}

proc define-register {name class attributes} {
   global  CurrentServer
   global EGallow

   set register /$CurrentServer/$name
   set-footer "defining register $register"
   set egallow $EGallow
   set EGallow 0x20006
   EG "register :name '$register' :class '$class' !new"
   set EGallow $egallow
   EG "register :name '$register' :object '$attributes'"
   clear-footer
}

proc delete-register {name} {
   global  CurrentServer

   set register /$CurrentServer/$name
   set-footer "deleting register $register"
   EG "register :pattern '$register' !delete"
}

proc modify-register {name attributes} {
   global  CurrentServer

   set register /$CurrentServer/$name
   EG "register :name '$register' :object '$attributes'"
}

proc read-register {name} {
   global  CurrentServer

   set register /$CurrentServer/$name
   return [EG "register :name '$register' ?value"]
}

proc write-register {name value} {
   global  CurrentServer

   set register /$CurrentServer/$name
   return [EG "register :name '$register' :value '$value'"]
}

proc inquire-registers {pattern} {
   global  CurrentServer

   set register /$CurrentServer/$pattern
   return [EG "register :pattern '$register' ?names"]
}

proc make-frame {} {
   global dratedef dstatdata drateperiod stat_lines
   
   set col1 180
   set col2 100

   :frame [self frame-name] :label "Exogam Data Quality" :show-footer T :resize F
   
   :panel PANEL1 :layout T :padx 5 :pady 5
      :layout H; :row-gap 5; :col-gap 20
      
         :text DSTATP :label "Update period (secs)"  :value $drateperiod :valuex 24c :w 8
         :button DSTATC :label "Clear Statistics"

   :panel PANEL2 :layout L :padx 5 :pady 5
      :layout H; :row-gap 5; :col-gap 20
         :message MSG2C :label "Count" :valuex 29c :bold T
         :message MSG2R :label "Rate (/sec)" :valuex 14c :bold T
         :next-row T
         for {set i 4} {$i < $stat_lines} {incr i} {
            set dstatdata($i) 0
            :text L$i  :label $dratedef($i) :valuex $col1 :w 12 :deaf T
            :text R$i  :label ""  :w 8  :deaf T
            :next-row T
         }
         
   :panel PANEL3 :layout L :padx 5 :pady 5
      :layout H; :row-gap 5; :col-gap 20
         :message MSG3C :label "Count" :valuex 20c :bold T
         :message MSG3R :label "Rate (/sec)" :valuex 14c :bold T
         :next-row T
         for {set i 0} {$i < 4} {incr i} {
            set dstatdata($i) 0
            :text L$i  :label $dratedef($i) :valuex $col2 :w 12 :deaf T
            :text R$i  :label ""  :w 8  :deaf T
            :next-row T
         }  
           
   :show T         
}

proc update-dstats {} {
  global dratedef dstatdata d2vbstats stat_lines

  set z [catch {read-register $d2vbstats} l]
  if {$z != 0} then {
      set-footer "unable to access EB statistics block"
      return 1
  }

  for {set i 0} {$i < $stat_lines} {incr i} {
     set vl [lindex $l $i]
     :modify L$i :value $vl
     set dstatdata($i) $vl
  }
  return 0
}

set drateperiod 40
set dratesactive 0

proc update-drates {} {
  global  dratedef dstatdata drateperiod dratesactive stat_lines
  
  if {$dratesactive == 0} then {return}
  if {$drateperiod <= 0} then {return}
  
  if {[winfo ismapped .]} then {

     for {set i 0} {$i < $stat_lines} {incr i} {
        set old($i) $dstatdata($i)
     }
     if {[update-dstats] == 0 } then {
        for {set i 0} {$i < $stat_lines} {incr i} {
           set vl [expr ($dstatdata($i) - $old($i)) / $drateperiod]
           :modify R$i :value $vl
        }
        set-footer "Updated [date]"
     }
  }
  after [expr $drateperiod * 1000] update-drates
}

proc start-update {} {
  global dratesactive drateperiod
  
  update-dstats
  after cancel update-drates
  after [expr $drateperiod * 1000] update-drates
  set dratesactive 1
}

proc start {} {
  global dratesactive drateperiod

  make-frame
  
  start-update
}

proc start-drates {} {
   start-update
}

proc stop-drates {} {
  global dratesactive
  set dratesactive 0
}


#    procedures to handle user input

proc click-dstatp {v} {
   global drateperiod
   set drateperiod $v
}

proc click-dstatc {} {
   inform serv clear-D2VB-stats
}
