#    sys.tcl     ExoGam Version   /exogam/tcl/sys.tcl

set debug 0
set started 0

set frame SYS
set frametitle "ExoGam Experiment Control"
set icontitle  "ExptCntl"

set autoperiod 5000
set autocount 10

set tapestreams 1
set tapedrives 4

set showcontrols 0

if {$env(MASTERCONSOLE) == "ALL"  || $env(MASTERCONSOLE) == [exec hostname]} then {
   set VXIEnable 1
   set ACSEnable 1
   set TSEnable 1
} else {
   set VXIEnable 0
   set ACSEnable 0
   set TSEnable 0
}

set vxistate 0
set acsstate 0
set tapestate 0

set inputproc EXOGAMD2VB
set outputproc OUTPUT
set expname ExogamTest
set bufsize 0x4000
set outputtransmit 1
set d2vbtransmit 1

set acsstrings {"Idle" "Initialising" "Initialised"  "Starting" "Running" "Stopping" "Exiting" "Killing"}

#    list of menu items and associated actions for the ACTIONS menu button 

set actionlist {                                                    \
   {"Show Main Menu"                    "show_main_menu"}       \
   {"Make System Console"               "do_system_console"}    \
   {"Make Slave Console"                "do_slave_console"}     \
   {"Make configuration file"           "Make_Configuration_File"}  \
   {"CAEN Amplifier Setup"              "send caenamp start"}       \
   {"Toggle System Controls State"      "click-system"}             \
   {"Pause VXI"                         "click-vxipause 1"}             \
   {"Resume VXI"                        "click-vxipause 0"}             \
   {""      ""}             \
   {"VXI Reset"                         "click-vxi 3"}              \
   {"VXI Setup"                         "click-vxi 4"}              \
   {"Start Trigger rate monitoring"     "send rates start-trig-rates"}    \
   {"Stop Trigger rate monitoring"      "send rates stop-trig-rates"}    \
   {"Start Singles rate monitoring"     "send rates start-sing-rates"}    \
   {"Stop Singles rate monitoring"      "send rates stop-sing-rates"}    \
   {"Start I/O rate monitoring"         "send rates start-serv-rates"}    \
   {"Stop I/O rate monitoring"          "send rates stop-serv-rates"}    \
   {"Start Tape rate monitoring"        "send rates start-tape-rates"}    \
   {"Stop Tape rate monitoring"         "send rates stop-tape-rates"}    \
   {"Start Tape monitoring"             "send rates start-tapemonitor"}    \
   {"Stop Tape monitoring"              "send rates stop-tapemonitor"}    \
}

proc do_system_console {} {
   global VXIEnable ACSEnable TSEnable

   set VXIEnable 1
   set ACSEnable 1
   set TSEnable 1

   click-redisplay

   inform xm enableActions

      inform rates start

   inform rates set tapewarn 90
   inform rates set tapehalt 95

      inform rates start-trig-rates
      inform rates start-sing-rates
      inform rates start-surv-rates
      inform rates start-tape-rates
      inform rates close-frame

}

proc do_slave_console {} {
   global VXIEnable ACSEnable TSEnable

   set VXIEnable 0
   set ACSEnable 0
   set TSEnable 0

   click-redisplay

   inform xm disableActions

      inform rates start

   inform rates set tapewarn disabled
   inform rates set tapehalt disabled

      inform rates stop-trig-rates
      inform rates stop-sing-rates
      inform rates stop-surv-rates
      inform rates stop-tape-rates
      inform rates close-frame

}

proc Make_Configuration_File {} {
  global env

    inform vxicontrol autostart
    inform vxicontrol "Make_Configuration_File"
    set-footer "Exogam configuration file written to $env(EXPERIMENTDIR)"
}

proc show_main_menu {} {
   global env
#    see if security option enabled and check permissions if it is
   if {$env(MIDASSECURITY) != "disabled"} then {
      if {![inform security validate "bf-menu,c"]} then {
#    user does not have permission to open bf menu
         set msg "Access Denied\n\nA password is required to access the main menu and you have not supplied it"
         set z [catch {midas_dialog [self frame-name].warningDialog "Attempted Security Violation" \
           "$msg" "warning" 0 "OK" "supply password"} m]
         if {$z != 0} then {
            puts stderr "midas-warning: [self frame-name].warningDialog: $m"
            catch {destroy [self frame-name].warningDialog}
         } else {
            if {$m == 1} then {inform security start}
         }
         return "Access Denied"
      }
   }
   send bf open-frame
}

proc starttask {task} {
#    Check if it is necessary to pre-start server interface task

    set started [inform $task inquire started]
    if {$started == 0} then {
      send $task start
      send $task close-frame
    }
}

proc click-tapes {action} {
   global TSEnable

   if {$TSEnable == 0} then {return}

   starttask tape

#    set tape server state

   send tape click-egstate $action
   redisplay-tapes
   update_button_states
}

proc redisplay-tapes {} {
   global TSEnable tapestate showcontrols

   if {$TSEnable == 0} then {:modify TAPE :value -1; return}

   set-busy
   starttask tape

#    find tape server state

   catch {set tapestate [send tape read-egstate]}
   if {$showcontrols == 1} then {:modify TAPE :value $tapestate}
   :modify TSSTATE :value $tapestate

#    find run number

   catch {set run [fifth [send tape inquire-ganil-status]]}
   :modify TAPERUN :value $run

#    find data stream association

   catch {set tapeassoc [send tape find-assoc-strings 1]}
   :modify TAPESTRM :value "$tapeassoc"

   clear-busy
}


proc click-vxi {action args} {
    global VXIEnable

  if {$VXIEnable == 0} then {:modify VXI :value -1; return}

   starttask vxicontrol

    switch $action {
      0    {set p DAQ-halt}
      1    {set p DAQ-go}
      2    {set p DAQ-flush}
      3    {set p DAQ-reset}
      4    {set p DAQ-setup}
    }

   set-footer "sending $p to VXI control task"
   set-busy
   if {[llength $args] == 0} then {
       set z [catch {send vxicontrol $p} m]
   } else {
       set z [catch {send vxicontrol $p [first $args]} m]
   }
   clear-busy
   if {$m == ""} then {clear-footer} else {set-footer $m}
   redisplay-vxi
   update_button_states
   inform rates zero-rates
}

proc redisplay-vxi {} {
   global vxistate showcontrols
   global VXIEnable

  if {$VXIEnable == 0} then {return}

   set-busy
   starttask vxicontrol

   catch {set vxistate [send vxicontrol read-vxistate]}
   if {$showcontrols == 1} then {:modify VXI :value $vxistate}
   :modify VXISTATE :value $vxistate

   clear-busy
}

proc do-click-d2vb {action} {
   global ACSEnable

  if {$ACSEnable == 0} then {:modify ACSSTATE :value -1; return}

   set-busy

   starttask serv

   catch {send serv write-D2VB-command $action}

   after 1000

   clear-busy
   redisplay-dataacq
   update_button_states
}

proc click-acs {action} {
   global inputproc expname
    global ACSEnable

  if {$ACSEnable == 0} then {:modify ACSSTATE :value -1; return}

   set-busy

   starttask serv

    switch $action {
      0    {set c stopacq; set p "{}"}
      1    {set c startacq; set p "{}"}
      2    {set c exitacq; set p "{}"}
      3    {set c iniacq; set p "{}"}
      4    {set c stop; set p $inputproc}
      5    {set c start; set p $inputproc}
      6    {set c init; set p $inputproc}
      7    {set c stats_clear; set p $inputproc}
    }
# for now ACS doesn't pass these commands onto the D2VB (or anywhere else)
# so send them direct

   set-footer "sending $c to Serveur task"

   if {$action > 5} then {
      do-click-d2vb [expr $action - 4]
      return
   }
   if {$action == 3} then {
      catch {send serv click-sexpname $expname}
      catch {send serv do-initacq}
   } else {
      catch {send serv write-ACS-command $p $c}
   }

   after 1000

   clear-busy
   redisplay-dataacq
   update_button_states
}

proc click-input {} {
# toggle d2vbtransmit
# These lines are the correct way round since D2VB requires 0 when
# d2vbtransmit is 1 and vice-versa
   global d2vbtransmit inputproc
   :modify INPUT :value $d2vbtransmit
   set d2vbtransmit [expr $d2vbtransmit + 1 & 1]
   if {$d2vbtransmit} then {
      set z [catch {send serv do-setpr $inputproc "transmit"} v]
   } else {
      set z [catch {send serv do-unsetpr $inputproc "transmit"} v]
   }
}

proc click-output {} {
# toggle outputtransmit
# These lines are the correct way round since OUTPUT requires 0 when
# outputtransmit is 1 and vice-versa
   global outputtransmit outputproc
   :modify OUTPUT :value $outputtransmit
   set outputtransmit [expr $outputtransmit + 1 & 1]
   if {$outputtransmit} then {
      set z [catch {send serv do-setpr $outputproc "transmit"} v]
   } else {
      set z [catch {send serv do-unsetpr $outputproc "transmit"} v]
   }
}

proc redisplay-dataacq {} {
   global d2vbtransmit outputtransmit
   global inputproc outputproc acsstate showcontrols

   starttask serv

   set-busy

   set z [catch {send serv get-ACS-state 0} acsstate]
   if {!$z} then {
      if {$showcontrols == 1} then {
         set z [catch {send serv get-ACS-state 1} v]
         if {!$z} then {:modify DAQ :value $v}
         set z [catch {send serv is-transmitting $inputproc} v]
         if {!$z && $v == "0" || $v == "1"} then {
            :modify INPUT :value [set d2vbtransmit $v]
         }
         set z [catch {send serv is-transmitting $outputproc} v]
         if {!$z && $v == "0" || $v == "1"} then {
            :modify OUTPUT :value [set outputtransmit $v]
         }
      }
      :modify ACSSTATE :deaf F :value $acsstate :deaf T
   }

   clear-busy
}

proc click-redisplay {} {
  global VXIEnable ACSEnable TSEnable

  foreach subsystem {VXI ACS TS} {
       set v [eval set [set subsystem]Enable]
       :modify [set subsystem]REQ :value $v
       if {$v == 0} then {
         :modify [set subsystem]STATE :colours "white black" :value 0
       } else {
         :modify [set subsystem]STATE :colours "red green" :value 0
       }
  }

  update-rates

  :modify TAPESTRM :value "unknown"

  if {$VXIEnable == 1} then {redisplay-vxi}
  if {$ACSEnable == 1} then {redisplay-dataacq}
  if {$TSEnable == 1}  then {redisplay-tapes}

  update_button_states
}

proc update_button_states {} {
  global VXIEnable ACSEnable TSEnable
  global vxistate acsstate tapestate

#    if any one of the controlled subsystems is in the GO state then the
#    overall GO button should be disabled

  if {$VXIEnable == 1 && $vxistate == 1}     then {disable_go; return 1}
  if {$ACSEnable == 1 && $acsstate == 1}     then {disable_go; return 1}
  if {$TSEnable == 1 && $tapestate == 1}     then {disable_go; return 1}

  :modify EXGO :colour green :inactive F
  return 0
}

proc exogam_state {} {
  global VXIEnable ACSEnable TSEnable
  global vxistate acsstate tapestate

#    if any one of the controlled subsystems is in the GO state then the
#    overall state is GO

  set state 0

  if {$VXIEnable == 1 && $vxistate == 1} then {set state 1}
  if {$ACSEnable == 1 && $acsstate == 1} then {set state 1}
  if {$TSEnable == 1 && $tapestate == 1} then {set state 1}

  return $state
}


proc disable_go {} {:modify EXGO :colour white :inactive T}

proc update-rates {} {
  global tapestreams tapedrives TSEnable

    set rates [list 0 0 0 0 0 0 0 0]

    set-footer "Updating Rates"
    set names [inform rates inquire ScalerNames]
    set i 1
    foreach name $names {
      :modify RATE$i :label $name
      incr i
    }
    set rates [inform rates inquire triggerrates]
    foreach i {1 2 3 4 5 6 7 8} {
      :modify RATE$i :value [lindex $rates [expr $i - 1]]
    }

#    calculate deadtime -  warning this calculation is dependent on the trigger lca program

    set ftr1 [lindex $rates 0]
    set ftr2 [lindex $rates 1]
    set ftr3 [lindex $rates 2]
    set ftr4 [lindex $rates 3]
    set ft   [lindex $rates 4]

#    set ftrtotal [expr $ftr1 + $ftr2 + $ftr3 + $ftr4]
    set ftrtotal [expr $ftr1 + $ftr2 + $ftr3]
    set ftrlost  [expr $ftrtotal - $ft]

    if {$ftrtotal > 0} then {
       set deadtime [expr [expr $ftrlost * 100] / $ftrtotal]
    } else {
       set deadtime 0
    }
    :modify DEADTIME :value $deadtime
    :modify DTPC :label "[set deadtime]%"

    set names [inform rates get-singles-processors]
    set i 1
    foreach name $names {
      :modify SINGRATE$i :label [string toupper $name 0]
      incr i
    }
    set rates [inform rates inquire singlesrates]
    set i 1
    foreach name $names {
      if {$i < 3} then {:modify SINGRATE$i :value [lindex $rates [expr $i - 1]]}
      incr i
    }

    set names [inform rates inquire CounterNames]
    set i 1
    foreach name $names {
      :modify IPRATE$i :label $name
      incr i
    }
    set rates [inform rates inquire IPrates]
    foreach i {1 2 3} {
      :modify IPRATE$i :value [lindex $rates [expr $i - 1]]
    }

    set names [first [inform rates inquire CounterNames]]
    set i 1
    foreach name $names {
      :modify OPRATE$i :label $name
      incr i
    }
    set rates [inform rates inquire OPrates]
    foreach i {1} {
      :modify OPRATE$i :value [lindex $rates [expr $i - 1]]
    }

    set rates [inform rates inquire taperates]
    :modify TAPERATE :value [first $rates]

    if {$TSEnable == 1} then {:modify TAPEPERCENT :value [string range [inform tape inquire tapepercent] 0 3]}
 
    clear-footer
}

proc auto-rates {} {
   global autoperiod autocount
   after $autoperiod auto-rates
   incr autocount
   if {![winfo ismapped .]} then {return}
   if {$autocount < 10} then {update-rates} else {set autocount 0; click-redisplay}
}

proc click-exogam {action} {
  global env VXIEnable ACSEnable TSEnable

   switch $action {
      0   {
          insert-log "ExoGam Data Acquisition Stopped [date]"
          set-footer "Stopping ExoGam Data Acquisition [date]"
          if {$VXIEnable == 1} then {click-vxi 0}
          if {$ACSEnable == 1} then {click-acs 0}
          if {$TSEnable == 1} then {click-tapes 0}
          set-footer "ExoGam Data Acquisition Stopped [date]"
         }
      1  {
          set-footer "Starting ExoGam Data Acquisition [date]"

          if {$VXIEnable == 1 && [inform vxicontrol VXI-ResetState] == 0} then {
             set r [midas-confirmation "The VXI control software has NOT performed a software reset\n\
                for one or more VXI servers\nIt may recently have been restarted\n\
                Your VXI hardware may not operate correctly unless this is done.\n\
                Do you wish to continue or to abort so that you can SETUP first"]
             if {$r == 0} then {return -1}
          }

          if {$VXIEnable == 1 && [inform vxicontrol VXI-RestoreState] == 0} then {
             set r [midas-confirmation "You have not performed a parameter restore for one or more VXI servers\n\
                It may recently have been restarted\n\
                Do you wish to continue or to abort so that you can SETUP first"]
              if {$r == 0} then {return -1}
          }

#    Do tape action first - then if it fails we don't continue

          if {$TSEnable == 1} then {
             if {[send tape read-storagestate] != 0} then {
                set r [midas-confirmation "Tape Server is in No Storage state \n\
                     Do you wish to continue or to abort so that you can change this"]
                if {$r == 0} then {return -1}
             }
             click-tapes 1
             if {[send tape read-egstate] != 1} then {
                set r [midas-confirmation "Tape Server not in Going state n\
                     Do you wish to continue or to abort so that you can correct the problem"]
                if {$r == 0} then {return -1}
             }
          }

          if {$VXIEnable == 1} then {click-vxi 2}

# always try to reset the singles sharc links
          if {$VXIEnable == 1} then {inform singles initialise-links}

          if {$ACSEnable == 1} then {click-acs 1}
          if {$VXIEnable == 1} then {click-vxi 1}

          set-footer "ExoGam Data Acquisition Started [date]"
          insert-log "ExoGam Data Acquisition Started [date]"
      }
   }

   set footertext $env(FooterText)
   click-redisplay
   set-footer $footertext

   return [exogam_state]
}

proc write-vxi-register {register value} {EG "register :name '/VXI/$register' :value '$value'"}

proc click-setup {arg} {
    global VXIEnable ACSEnable TSEnable

if {$arg == 0} then {

#    allow the user to chicken out!

    set msg "\n\
Are you sure you wish to SETUP the data acquisition system?\n\n\
Only those Subsystems selected for control (VXI; ACS etc) will be accessed regardless of your SETUP option.\n\n\
SETUP will STOP data acquisition\n\
Configure the VXI Control software\n\
Software RESET the VXI hardware\n\
Software SETUP the VXI hardware\n\
Restore VXI parameters\n\
and finally restart the Acquisition readout software.\n\
This may take several minutes.\n\n\
You can choose that RESET and SETUP operations are only performed if the software thinks they are needed or you can choose that they are peformed everywhere.\n\
If you are not sure you can choose to be prompted.\n\n
If you decide to continue please wait for the SETUP sequence to complete and DO NOT touch the system while it is in progress.
"

    set z [catch {midas_dialog [self frame-name].confirmDialog "Confirmation Request for Data Acquisition SETUP" \
           "$msg" "" 1 "Cancel" "Setup only where needed" "Setup everything"  "Prompt me"} m]
    if {$z != 0} then {
       puts stderr "midas-warning: [self frame-name].confirmDialog: $m"
       destroy [self frame-name].confirmDialog
       return 0
    }

    if {$m == 0} then {return}
    set opt [expr $m - 1]

} else {
    set opt 0
}

  set-footer "Stopping ExoGam Data Acquisition [date]"

  if {$VXIEnable != 0} then {
     if {$opt != 2} then {
        if {[inform vxicontrol inquire VXIconfigured] == 0} then {inform vxicontrol "configure-all  1"}
     } else {
        if {[inform vxicontrol inquire VXIconfigured] != 0} then {
            set r [midas-confirmation "The VXI control software has already been configured and so this does not need to be repeated.\n\
               However if you have recently made changes to the VXI hardware installed the VXI control software should probe the hardware \
               to update its configuration information." \
               "Probe for VXI hardware changes?"  "Skip to next step (recommended)"]
            if {$r == 0} then {inform vxicontrol "configure-all  1"}
        } else {
            set r [midas-confirmation "The VXI control software has NOT been configured and so is uncertain about your VXI hardware.\n\
               The VXI control software should probe the hardware to obtain the configuration information." \
               "Probe for VXI hardware changes? (recommended)"  "Skip to next step"]
            if {$r == 0} then {inform vxicontrol "configure-all  1"}
        }
     }
     inform vxicontrol "Stop-VXI VXI"
  }

#    stopping DA
    click-exogam 0

   if {$VXIEnable != 0} then {

#   reset VXI

     if {$opt != 2} then {
        if {[inform vxicontrol VXI-ResetState] == 0 || $opt == 1} then {click-vxi 3} else {click-vxi 3 1}
     } else {
        if {[inform vxicontrol VXI-ResetState] != 0} then {
            set r [midas-confirmation "The VXI control software has already performed a software reset and so this does not need to be repeated.\n\
               You may repeat the software reset if you wish but this may cause VXI module setup information to be lost." \
               "Perform software reset?"  "Skip to next step (recommended)"]
            if {$r == 0} then {click-vxi 3}
        } else {
            set r [midas-confirmation "The VXI control software has NOT performed a software reset.\n\
               Your VXI hardware may not operate correctly unless this is done." \
               "Perform software reset? (recommended)"  "Skip to next step"]
            if {$r == 0} then {click-vxi 3}
        }
     }

     if {[inform vxicontrol VXI-ResetState] == 0} then {
         midas-warning "VXI software reset has not been successful\nYou may need to correct this and retry SETUP"
     }

#    setup VXI hardware

     if {$opt != 2} then {
        if {[inform vxicontrol VXI-SetupState] == 0 || $opt == 1} then {click-vxi 4} else {click-vxi 4 1}
     } else {
        if {[inform vxicontrol VXI-SetupState] != 0} then {
            set r [midas-confirmation "The VXI control software has already performed a software setup and so this does not need to be repeated.\n\
               You may repeat the software setup if you wish but this may cause VXI module setup information to be lost." \
               "Perform software setup?"  "Skip to next step (recommended)"]
            if {$r == 0} then {click-vxi 4}
        } else {
            set r [midas-confirmation "The VXI control software has NOT performed a software setup.\n\
               Your VXI hardware may not operate correctly unless this is done." \
               "Perform software setup? (recommended)"  "Skip to next step"]
            if {$r == 0} then {click-vxi 4}
        }
     }

     if {[inform vxicontrol VXI-SetupState] == 0} then {
         midas-warning "VXI software setup has not been successful\nYou may need to correct this and retry SETUP"
     }

#   restore VXI parameters

     set-busy

     set r 1
     if {$opt != 2} then {
        if {[inform vxicontrol VXI-RestoreState] == 0 || $opt == 1} then {
            set r 0
            if {$opt == 0} then {
                inform vxirestore "click-restore-current 1"
            } else {
                inform vxirestore "click-restore-current 0"
            }
        }
     } else {
        if {[inform vxicontrol VXI-RestoreState] != 0} then {
            set r [midas-confirmation "The VXI control software has already performed a parameter restore and so this does not need to be repeated.\n\
               You may repeat the parameter restore if you wish or because you suspect that the parameters may have become corrupted.\n\
               If you choose to perform a restore parameters will be taken from the current save for each VXI crate."\
               "Perform parameter restore?"  "Skip to next step (recommended)"]
            if {$r == 0} then {inform vxirestore "click-restore-current 0"}
        } else {
            set r [midas-confirmation "The VXI control software has NOT performed a parameter restore.\n\
               Your VXI hardware may not operate correctly unless this is done.\n\
               If you choose to perform a restore parameters will be taken from the current save for each VXI crate."\
               "Perform parameter restore? (recommended)"  "Skip to next step"]
            if {$r == 0} then {inform vxirestore "click-restore-current 0"}
        }
     }
     if {$r == 0} then {inform  vxirestore destroy  .vxirestore.info}

#    reload the VRE firmware which may have been changed
     if {$r == 0} then {inform vre vre-reloadDSP VXI}
    
     if {[inform vxicontrol VXI-RestoreState] == 0} then {
        midas-warning "VXI parameter restore has not been successful\nYou may need to correct this and retry SETUP"
     }

     clear-busy
   }

#    setup readout software

    if {$ACSEnable == 1} then {
#      exit acquisition processes
       set-footer "Exiting Acquisition Processes"
       click-acs 2
    
#      reinitialise acquisition processes
       set-footer "Initialising Acquisition"
       click-acs 3
    
#      init Data Acquisition
#       set-footer "Initialising Output Process"
#       click-acs 6
    
#      setup Data Acquisition
#       set-footer "Clearing Output Statistics"
#       click-acs 7
    }

    if {$opt == 2} then {
        midas-report "ExoGam Data Acquisition System SETUP and ready to GO at [date]"
    }

    insert-log "ExoGam Data Acquisition System SETUP and ready to GO at [date]"
    set-footer "ExoGam DA System SETUP and ready to GO at [date]"        
}

proc click-vxipause {opt} {
        switch $opt {
          1  {inform vxicontrol "Stop-VXI VXI"}
          0  {inform vxicontrol "Go-VXI  VXI"}
        }
    redisplay-vxi
    update_button_states
}

proc click-system {} {
    global showcontrols
    set showcontrols [expr $showcontrols + 1 & 1]
    make-frame
    click-redisplay
}


proc click arg {
        global debug
        global VXIEnable ACSEnable TSEnable
        global actionlist

        if {$debug} then {puts stderr "click with argument \{$arg\}"}

        set f [lindex $arg 0]
        set c [lindex $arg 1]
        set w [lindex $arg 2]
        set v [lindex $arg 3]
        set i [lindex $arg 4]
        if {$c==":quit"} then {click-quit; return}
        switch -glob $w {
                VXI      {click-vxi $v}
                INPUT    {click-input}
                OUTPUT   {click-output}

                TAPE  {click-tapes $v}
                SPECTRA {inform sb start; inform sd start}

                VXIREQ  {set VXIEnable $v; click-redisplay}
                ACSREQ  {set ACSEnable $v; click-redisplay}
                TSREQ   {set TSEnable $v; click-redisplay}

                ACTIONS  {
                    set action [lindex $actionlist $i]
                    set r 1
                    if {$i > 0} then {
                       set r [midas-confirmation "You are about to execute the action \"[first $action]\"\nAre you sure?"]
                    }
                    if {$r == 1} then {eval [second $action]}
                }

                default {puts stderr "sys:  click \{$arg\}"}
        }
}

proc start args {
   global started env
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   starttask vxicontrol
   starttask vxirestore
   if {$env(MASTERCONSOLE) == "ALL"  || $env(MASTERCONSOLE) == [exec hostname]} then {
      inform rates start
      inform rates start-trig-rates
      inform rates start-sing-rates
      inform rates start-surv-rates
      inform rates start-tape-rates
      inform rates close-frame
   }
   click-redisplay
   auto-rates
}

proc click-quit args {close-frame; send bf click-quit}

proc finish args {destroy-frame}

proc frame-quit win  {
   catch {wm iconify [winfo toplevel $win]}
   wm withdraw [winfo toplevel $win]
   send bf click-quit
}

proc make-frame {} {
  global env tapestreams tapedrives showcontrols
  global VXIEnable ACSEnable TSEnable
  global vxistate acsstate tapestate
  global actionlist acsstrings

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/egarray" \
      :icon-name "[self icon-title]" :resize F

  if {$showcontrols == 1} then {
    :panel PANEL1 :layout T  :padx 100 :pady 10
     :layout V; :col-gap 10; :row-gap 10
      :choice VXI      :label "VXI Electronics:"           :strings [list Stop Go Flush Reset Setup] :valuex 18c
      :choice DAQ      :label "Data Acquisition State:"    :strings $acsstrings :valuex 18c :deaf T
      :choice INPUT    :label "Input Process (D2VB):"      :strings {"Transmit Data"} :valuex 30c :value 1
      :choice OUTPUT   :label "Output Process:"            :strings {"Transmit Data"} :valuex 30c :value 1
      :choice TAPE     :label "Tapes:"                     :strings [list Stop Go Test]        :valuex 18c
  }

   :panel PANEL2A :layout L :padx 0 :pady 5
     :layout V; :col-gap 10; :row-gap 10
       :message MSG1A :label "Subsystems Controlled:" :bold T
         :checkbox VXIREQ     :label "VXI Electronics"   :valuex 15c  :value $VXIEnable
         :checkbox ACSREQ     :label "Data Acquisition"  :valuex 15c  :value $ACSEnable
         :checkbox TSREQ      :label "Tapes"             :valuex 15c  :value $TSEnable 

   :panel PANEL2B :layout L :padx 0 :pady 5
     :layout V; :col-gap 10; :row-gap 8
       :message MSG1B :label "Open" :bold T
         :button VXICTRL      :label "  " :command "inform vxicontrol start"
        :vskip -2
         :button ACSCTRL      :label "  " :command "inform serv start"
        :vskip -4
         :button TAPECTRL     :label "  " :command "inform tape start"  

   :panel PANEL2C :layout L :padx 0 :pady 5
     :layout V; :col-gap 10; :row-gap 10
       :message MSGEX    :label " EXOGAM " :bold T :colour "-background black -foreground white"
       :button EXGO      :label "GO"     :w 8  :h 2 :colour green  :command "click-exogam 1"
       :button EXSTOP    :label "STOP"   :w 8  :h 2 :colour red    :command "click-exogam 0"
       :button EXSETUP   :label SETUP    :w 8  :h 1 :colour yellow :command "click-setup 0"

   :panel PANEL2D :layout L :padx 0 :pady 5
     :layout V; :col-gap 10; :row-gap 10
       :message MSG2 :label "Subsystem States:" :bold T
         :checkbox VXISTATE :label "VXI Electronics"   :valuex 14c  :value $vxistate   :colours "white black" :deaf T
         :checkbox ACSSTATE :label "Data Acquisition"  :valuex 14c  :value $acsstate   :colours "white black" :deaf T
         :checkbox TSSTATE  :label "Tapes"             :valuex 14c  :value $tapestate  :colours "white black" :deaf T

   :panel PANEL3 :layout B :padx 5 :pady 5 :after FOOTER
     :layout H; :col-gap 20; :row-gap 10

      :text TAPERUN     :label "Current Tape Run Number" :w 8 :deaf T  :value "unknown" 
      :next-row T
      :text TAPERATE    :label "Tape Rate (Kbytes/sec)" :w 6 :deaf T
      :text TAPEPERCENT :label "Tape Used (%)" :w 6 :deaf T
      :next-row T
      :text TAPESTRM    :label "Drives in Use" :w 50 :deaf T

   :panel PANEL4 :layout B :padx 5 :pady 5 :after FOOTER
     :layout H; :col-gap 0; :row-gap 5
      :message MSG8 :label "Output Process Rates (/sec)" :bold T
       :next-row T
       :text OPRATE1  :label "" :w 8 :valuex 8c :deaf T
       :next-row T
      :message MSG9 :label "Input Process Rates (/sec)" :bold T
       :next-row T
       :text IPRATE1  :label "" :w 8 :valuex 8c :deaf T
       :text IPRATE2  :label "" :w 8 :valuex 12c :deaf T
       :text IPRATE3  :label "" :w 8 :valuex 12c :deaf T
       :next-row T
      :message MSG10 :label "Singles Rates (/sec)" :bold T
       :next-row T
       :text SINGRATE1  :label "" :w 8 :valuex 8c :deaf T
       :text SINGRATE2  :label "" :w 8 :valuex 12c :deaf T
       :next-row T
      :message MSG4 :label "Trigger Scaler Rates (/sec)" :bold T
       :next-row T
       :text RATE1  :label "" :w 8 :valuex 5c :deaf T
       :text RATE2  :label "" :w 8 :valuex 5c :deaf T
       :text RATE3  :label "" :w 8 :valuex 5c :deaf T
       :text RATE4  :label "" :w 8 :valuex 5c :deaf T
       :next-row T
       :text RATE5  :label "" :w 8 :valuex 5c :deaf T
       :text RATE6  :label "" :w 8 :valuex 5c :deaf T
       :text RATE7  :label "" :w 8 :valuex 5c :deaf T
       :text RATE8  :label "" :w 8 :valuex 5c :deaf T
       :next-row T
       :row-gap 0
      :message MSG7 :label "Trigger Requests not handled" :bold T
      :message DTPC :label "(0%)" :bold T
       :next-row T; :hskip 25
      :gauge DEADTIME :label "" :min 0 :max 100 :gauge-width 400 :gauge-height 10 :inactive T 
      [:path DEADTIME].scale configure -troughcolor yellow -background blue -sliderlength 20

   :panel BUTTONS :layout B :padx 5 :pady 5 :after FOOTER
     :layout H; :col-gap 10; :row-gap 0
      :message MSG3 :label "Controls" :bold T
      :next-row T
      :row-gap 10
      :button VXIMSETUP  :label "VXI Module Setup"  :command "inform vxisetup start"
      :button HISTOGRAM  :label "Histogrammer"      :command "inform hist start"
      :button SPECTRA    :label "Spectrum Viewer"
      :button PRINT      :label "Print"             :command "midas-capture window"
      :next-row T
      :button VXIRESTORE    :label "VXI Parameter Restore"  :command "inform vxirestore start"
      :button VXISAVE       :label "VXI Parameter Save"     :command "inform vxisave start"
      :button SINGLES       :label "Singles"                :command "inform singles start"
     :next-row T; :col-gap 10
      :button REDISPLAY   :label "Redisplay"             :command "click-redisplay"
      :button STATISTICS  :label "Statistics"            :command "inform stats start"
      :button RATES       :label "Rates and Monitoring"  :command "inform rates start"

      set menu {}
      foreach item $actionlist {lappend menu [first $item]}
      :menubutton ACTIONS :label "More Actions"  :strings $menu

#      :button HELP        :command click-help

   :show T
}

loadsource endrun+newrun

